/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.console.webui.novaview;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.webui.core.UIComponent;
import org.exoplatform.webui.event.Event;
import org.exoplatform.webui.event.EventListener;

/**
 * The nova view tabulation is a container of views with special purpose which
 * is to manage two lists. A list of visible tabulations and another of hidden
 * ones.
 * 
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 * @author Elwalid BRAHIM <elwalid.brahim@bull.net>
 */
public class UINovaViewTabulation extends UINovaViewContainer {

  /**
   * Maximum number of visible tabulations
   */
  private int visibleSize = 4;

  /**
   * The list of the hidden contained views
   */
  private List<UIComponent> hiddenChildren;

  /**
   * Check if the nova view tabulation has some hidden views
   * 
   * @return boolean
   */
  public boolean hasHiddenChildren() {
    return hiddenChildren != null;
  }

  /**
   * Get the list of hidden views
   * 
   * @return list of views
   */
  public List<UIComponent> getHiddenChildren() {
    if (hiddenChildren == null) {
      hiddenChildren = new ArrayList<UIComponent>(3);
    }
    return hiddenChildren;
  }

  /**
   * Add a child (a view) to the view tabulation
   * 
   * @param ui component
   */
  public void addChild(UIComponent component) {
    if (this.getChildren().size() < this.visibleSize) {
      super.addChild(component);
    } else {
      addHiddenChild(component);
    }
  }

  @SuppressWarnings("unchecked")
  public <T extends UIComponent> T getChildById(String id) {
    UIComponent child = super.getChildById(id);
    if (child == null) {
      for (UIComponent ui : this.hiddenChildren) {
        if (ui.getId().equals(id)) {
          return (T) ui;
        }
      }
    }
    return (T) child;
  }

  /**
   * Remove a child with a given id
   * 
   * @param child id
   * @return the remove ui component
   */
  @SuppressWarnings("unchecked")
  public <T extends UIComponent> T removeChildById(String id) {
    UIComponent c = super.removeChildById(id);
    if (c != null) {
      if (hiddenChildren != null && hiddenChildren.size() > 0) {
        super.addChild(hiddenChildren.remove(0));
      }
      return (T) c;
    }
    if (hiddenChildren == null) {
      return null;
    }
    for (UIComponent child : hiddenChildren) {
      if (!id.equals(child.getId())) {
        continue;
      }
      child.setParent(null);
      hiddenChildren.remove(child);
      return (T) child;
    }
    return null;
  }

  /**
   * Remove a child with a given class
   * 
   * @param class of the child to remove
   * @return the removed ui component
   */
  public <T extends UIComponent> T removeChild(Class<T> clazz) {
    T c = super.removeChild(clazz);
    if (c != null) {
      return c;
    }
    if (hiddenChildren == null) {
      return null;
    }
    for (UIComponent child : hiddenChildren) {
      if (!clazz.isInstance(child)) {
        continue;
      }
      child.setParent(null);
      hiddenChildren.remove(child);
      return clazz.cast(child);
    }
    return null;
  }

  /**
   * This method allow to make visible a nova view according to the given view
   * id. If the view is included in the hidden views list, it is deleted from
   * this list and appear with the visible views list.
   * 
   * @param view id
   */
  public void switchView(String viewId) {
    if (isSwitchView()) {
      switchVisibleView(viewId);
      switchHiddenView(viewId);
    }
  }

  /**
   * This method make visible the view, represented by the given view id.
   * 
   * @param view id
   */
  private void switchVisibleView(String viewId) {
    List<UIComponent> list = this.getChildren();
    Iterator<UIComponent> ite = list.iterator();
    while (ite.hasNext()) {
      UIComponent component = ite.next();
      if (component.getId().equals(viewId)) {
        component.setRendered(true);
      } else {
        component.setRendered(false);
      }
    }
  }

  /**
   * This method move the view, if it is a hidden one, from the hidden list to
   * the visible views list and make it visible.
   * 
   * @param view id
   */
  private void switchHiddenView(String viewId) {
    List<UIComponent> list = this.getHiddenChildren();
    List<UIComponent> toSwitch = new ArrayList<UIComponent>();
    Iterator<UIComponent> ite = list.iterator();
    while (ite.hasNext()) {
      UIComponent component = ite.next();
      if (component.getId().equals(viewId)) {
        component.setRendered(true);
        // The component need to be moved
        toSwitch.add(component);
      } else {
        component.setRendered(false);
      }
    }
    // Move the components
    ite = toSwitch.iterator();
    while (ite.hasNext()) {
      UIComponent component = ite.next();
      this.hiddenChildren.remove(component);
      this.hiddenChildren.add(0, component);
      if (this.getChildren().size() > 0) {
        addHiddenChild(this.removeChildById(((UIComponent) this.getChild(0)).getId()));
      }
      // TODO valider que cette condition est toujours fausse et supprimer
      if (this.getChildren().size() < this.visibleSize) {
        this.hiddenChildren.remove(component);
        super.addChild(component);
      }
    }
  }

  /**
   * Add the given component to the hidden views list
   * 
   * @param ui component
   */
  private void addHiddenChild(UIComponent uicomponent) {
    if (hiddenChildren == null) {
      hiddenChildren = new ArrayList<UIComponent>(3);
    }
    uicomponent.setParent(this);
    hiddenChildren.add(uicomponent);
  }

  /**
   * Set the maximum number of visible tabulation
   * 
   * @param size
   */
  public void setVisibleSize(int size) {
    this.visibleSize = size;
  }

  /**
   * Get the maximun number of visible actions
   * 
   * @return
   */
  public int getVisibleSize() {
    return this.visibleSize;
  }

  /**
   * This subclass is the listener of the close all views request event. It get
   * get all the views contained in the view tabulation and close them.
   */
  public static class CloseAllViewActionListener extends EventListener<UINovaViewTabulation> {

    /**
     * The execute method of the listener
     * 
     * @throws Exception
     */
    public void execute(Event<UINovaViewTabulation> event) throws Exception {
      UINovaViewTabulation uiContainer = event.getSource();
      UINovaViewManagement viewMgt = uiContainer.getAncestorOfType(UINovaViewManagement.class);
      List<String> ids = new ArrayList<String>();
      for (UIComponent child : uiContainer.getChildren()) {
        ids.add(child.getId());
      }
      for (UIComponent child : uiContainer.getHiddenChildren()) {
        ids.add(child.getId());
      }
      for (String id : ids) {
        viewMgt.closeView(id);
      }
      viewMgt.addContainerToUpdateByAjax(event.getRequestContext());
      event.getRequestContext().addUIComponentToUpdateByAjax(viewMgt.getViewer());
    }
  }
}
